import {
  Expression,
  NumberExpression,
  NumberExpressionConverter,
  StringExpression,
  StringExpressionConverter,
} from "adaptive-expressions";
import { Count } from "adaptive-expressions/lib/builtinFunctions";

import {
  Converter,
  ConverterFactory,
  Dialog,
  DialogConfiguration,
  DialogContext,
  DialogTurnResult,
} from "botbuilder-dialogs";

import { MicrosoftAppCredentials } from "botframework-connector";
import { AttachmentActions } from "./actions/attachmentActions";

export interface BotFrameworkAttachmentCustomActionsConfiguration extends DialogConfiguration {
  url: string | string | Expression | StringExpression;
  contentType: string | string | Expression | StringExpression;
  action: string | string | Expression | StringExpression;
  channelId: string | string | Expression | StringExpression;
  msAppId: string | string | Expression | StringExpression;
  msAppPwd: string | string | Expression | StringExpression;
  resultProperty?: string | Expression | StringExpression;
}

export class BotFrameworkAttachmentCustomActions
  extends Dialog
  implements BotFrameworkAttachmentCustomActionsConfiguration {
  public static $kind = "BotbuilderSamples.BotFrameworkAttachmentCustomActions";

  public url: StringExpression = new StringExpression('');
  public contentType: StringExpression = new StringExpression('');
  public action: StringExpression = new StringExpression('');
  public channelId: StringExpression = new StringExpression('');
  public msAppId: StringExpression = new StringExpression('');
  public msAppPwd: StringExpression = new StringExpression('');
  public resultProperty?: StringExpression;

  public getConverter(
    property: keyof BotFrameworkAttachmentCustomActionsConfiguration
  ): Converter | ConverterFactory {
    switch (property) {
      case "url":
        return new StringExpressionConverter();
      case "contentType":
        return new StringExpressionConverter();
      case "action":
        return new StringExpressionConverter();
      case "channelId":
        return new StringExpressionConverter();
      case "msAppId":
        return new StringExpressionConverter();
      case "msAppPwd":
        return new StringExpressionConverter();
      case "resultProperty":
        return new StringExpressionConverter();
      default:
        return super.getConverter(property);
    }
  }

  public async beginDialog(dc: DialogContext): Promise<DialogTurnResult> {
    const action = this.action.getValue(dc.state);
    let result;
    let debugStr = "|| 1 || ";
    try {
      if (action === 'GET_BASE64_CONTENT') {
        let token: string = '';
        let url = this.url.getValue(dc.state), urlStr: string = "";
        if (typeof url !== "string") {
          urlStr = JSON.stringify(url);
        } else {
          urlStr = url;
        }
        debugStr = debugStr + " || 1.1 ||";
        const channelId = this.channelId.getValue(dc.state);
        debugStr = debugStr + " || 2 || channelId: " + channelId;
        if (channelId == 'msteams' || channelId == 'skype') {
          debugStr = debugStr + " || 3 || app id: " + this.msAppId.getValue(dc.state);
          const credentials = new MicrosoftAppCredentials(this.msAppId.getValue(dc.state), this.msAppPwd.getValue(dc.state));
          debugStr = debugStr + " || 3.1 || b4 getToken!";
          token = await credentials.getToken(false);
        }
        debugStr = debugStr + " || 4 || urlStr: " + urlStr;
        result = await AttachmentActions.getBase64Content(urlStr, this.contentType.getValue(dc.state), token);
      }

      debugStr = debugStr + " || 5 || b4 resultProperty";
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), result);
      }
    } catch (error) {
      let errorText = "Unknown"; // error under useUnknownInCatchVariables 
      if (typeof error === "string") {
        errorText = error.toUpperCase() // works, `e` narrowed to string
      } else if (error instanceof Error) {
        errorText = error.message // works, `e` narrowed to Error
      }
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), JSON.stringify({
          "status": "FAILED-1",
          "url" : this.url.getValue(dc.state),
          "error": errorText,
          "debugLog": debugStr
        }));
      }
    }

    return dc.endDialog(result);
  }

  protected onComputeId(): string {
    return "BotbuilderSamples.BotFrameworkAttachmentCustomActions";
  }
}
